/*
 *  Extension to quickly show and modify the proxy used by the system
    Copyright (C) 2014  Jairo A Medina

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    jam-Proxy-Switcher@interhosting.us

    This program is loosely based on the Proxy Switcher code written by flannable 

*/
const St = imports.gi.St;
const Main = imports.ui.main;
const Gio = imports.gi.Gio;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const Lang = imports.lang;

const PROXY_SCHEMA = "org.gnome.system.proxy"
const PROXY_MODE = "mode"
const PROXY_NONE_TEXT = "None"
const PROXY_MANUAL_TEXT = "Manual"
const PROXY_AUTO_TEXT = "Automatic"
const ICON_NONE = "preferences-system-network-proxy-symbolic"
const ICON_MANUAL = "preferences-system-network-proxy-symbolic"
const ICON_AUTO = "preferences-system-network-proxy-symbolic"

function getSettings(schema) {
    if (Gio.Settings.list_schemas().indexOf(schema) == -1)
        throw _("Schema \"%s\" not found.").format(schema);
    return new Gio.Settings({ schema: schema });
}

function ProxySwitcherPanelButton() {
    this._init();
}

ProxySwitcherPanelButton.prototype = {
    __proto__: PanelMenu.Button.prototype,
    
    _init: function() {
        // connect to the gsettings proxy schema
        this._settings = getSettings(PROXY_SCHEMA);
	// read current mode from network settings
        this._mode = this._settings.get_string(PROXY_MODE);
        this._range = this._settings.get_range(PROXY_MODE);
        let load_settings_refresh = Lang.bind(this, function() {
            this._mode = this._settings.get_string(PROXY_MODE);
            this.refresh();
        });
        this._settings_connection_id = 
            this._settings.connect('changed::' + PROXY_MODE, 
                                   load_settings_refresh);
        
        this._icon = new St.Icon({ 
            icon_name: ICON_NONE,
            style_class: 'system-status-icon' });
        this._modeInfo = new St.Label({ text: "", width: 16 });

        // Panel menu item - the current class
        let menuAlignment = 0.25;
        PanelMenu.Button.prototype._init.call(this, menuAlignment);
        
        // Putting the panel item together
        let topBox = new St.BoxLayout();
        topBox.add_actor(this._icon);
        topBox.add_actor(this._modeInfo);
        this.actor.add_actor(topBox);

        let children = null;
        children = Main.panel._rightBox.get_children();
        Main.panel._rightBox.insert_child_at_index(this.actor, 0);

	// add menues for each available proxy mode
	// TODO read available modes from network settings and set up dynamic menues
	this._proxySettingMenuItemNone = new PopupMenu.PopupMenuItem(
    	    PROXY_NONE_TEXT);
        this.menu.addMenuItem(this._proxySettingMenuItemNone);
        this._proxySettingMenuItemNone.connect(
            'activate', Lang.bind(this, function() {
                this._settings.set_string(
                    PROXY_MODE, 'none');
            }));
        this._proxySettingMenuItemNone.setOrnament(this._mode == 'none' ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);

	this._proxySettingMenuItemManual = new PopupMenu.PopupMenuItem(
    	    PROXY_MANUAL_TEXT);
        this.menu.addMenuItem(this._proxySettingMenuItemManual);
        this._proxySettingMenuItemManual.connect(
            'activate', Lang.bind(this, function() {
                this._settings.set_string(
                    PROXY_MODE, 'manual');
            }));
        this._proxySettingMenuItemManual.setOrnament(this._mode == 'manual' ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);

	this._proxySettingMenuItemAuto = new PopupMenu.PopupMenuItem(
    	    PROXY_AUTO_TEXT);
        this.menu.addMenuItem(this._proxySettingMenuItemAuto);
        this._proxySettingMenuItemAuto.connect(
            'activate', Lang.bind(this, function() {
                this._settings.set_string(
                    PROXY_MODE, 'auto');
            }));
        this._proxySettingMenuItemAuto.setOrnament(this._mode == 'auto' ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);
        // Add a menu seperator and link to network settings to bottom of menu
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this.menu.addSettingsAction(_("Network Settings"), 
                                    'gnome-network-panel.desktop');
	// TODO: send DBUS message to Netowk settings to switch to proxy settings
	// this.sendmessage('gnome-network-panel.desktop','network-proxy')
//const PowerManagerProxy = Gio.DBusProxy.makeProxyWrapper(PowerManagerInterface);
//
        this.refresh();
    },
    
    refresh: function() {
        // run every time settings have changed.
        switch (this._mode) {
        case 'none':
            this._icon.icon_name = ICON_NONE;
            this._modeInfo.text = "-";
            this._proxySettingMenuItemNone.setOrnament(PopupMenu.Ornament.CHECK);
            this._proxySettingMenuItemManual.setOrnament(PopupMenu.Ornament.NONE);
            this._proxySettingMenuItemAuto.setOrnament(PopupMenu.Ornament.NONE);
            break;
        case 'manual':
            this._icon.icon_name = ICON_MANUAL;
            this._modeInfo.text = "m";
            this._proxySettingMenuItemNone.setOrnament(PopupMenu.Ornament.NONE);
            this._proxySettingMenuItemManual.setOrnament(PopupMenu.Ornament.CHECK);
            this._proxySettingMenuItemAuto.setOrnament(PopupMenu.Ornament.NONE);
            break;
        case 'auto':
            this._icon.icon_name = ICON_AUTO;
            this._modeInfo.text = "a";
            this._proxySettingMenuItemNone.setOrnament(PopupMenu.Ornament.NONE);
            this._proxySettingMenuItemManual.setOrnament(PopupMenu.Ornament.NONE);
            this._proxySettingMenuItemAuto.setOrnament(PopupMenu.Ornament.CHECK);
            break;            
        default:
            break;
        }
        //this._proxySettingMenuItemNone.setOrnament(this._mode == 'none' ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);
        //this._proxySettingMenuItemManual.setOrnament(this._mode == 'manual' ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);
        //this._proxySettingMenuItemAuto.setOrnament(this._mode == 'auto' ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);
    }
};

let proxySwitcherPanel;

function init() {

}

function enable() {
    proxySwitcherPanel = new ProxySwitcherPanelButton();
    Main.panel.addToStatusArea('proxySwitcherPanel', proxySwitcherPanel);
}

function disable() {
    proxySwitcherPanel._settings.disconnect(proxySwitcherPanel._settings_connection_id);
    proxySwitcherPanel.destroy();
}
